@@ -60,7 +60,7 @@ use std::sync::Arc;
 
 use chrono::naive::NaiveDate;
 use chrono::offset::Utc;
-use chrono::DateTime;
+use chrono::{DateTime, Duration};
 use rand::distributions::Alphanumeric;
 use rand::{thread_rng, Rng};
 use reqwest::{header, Client, Method, Request, StatusCode, Url};
@@ -709,7 +709,7 @@ impl GSuite {
     /// List calendars for a user.
     pub async fn list_calendars(&self) -> Result<Vec<Calendar>, APIError> {
         // Build the request.
-        let request = self.request(CALENDAR_ENDPOINT, Method::GET, "users/me/calendarList", (), None);
+        let request = self.request(CALENDAR_ENDPOINT, Method::GET, "users/me/calendarList", (), Some(&[("maxResults", "250")]));
 
         let resp = self.client.execute(request).await.unwrap();
         match resp.status() {
@@ -728,6 +728,41 @@ impl GSuite {
         Ok(value.items)
     }
 
+    /// List events on a calendar with a query.
+    pub async fn list_calendar_events_query(&self, calendar_id: &str, query: &str) -> Result<Vec<CalendarEvent>, APIError> {
+        // Build the request.
+        let request = self.request(
+            CALENDAR_ENDPOINT,
+            Method::GET,
+            &format!("calendars/{}/events", calendar_id),
+            (),
+            Some(&[
+                ("singleEvents", "false"),
+                ("maxResults", "2500"),
+                ("showDeleted", "true"),
+                ("q", query),
+                // This is one week into the future.
+                ("timeMax", &Utc::now().checked_add_signed(Duration::weeks(1)).unwrap().to_rfc3339()),
+            ]),
+        );
+
+        let resp = self.client.execute(request).await.unwrap();
+        match resp.status() {
+            StatusCode::OK => (),
+            s => {
+                return Err(APIError {
+                    status_code: s,
+                    body: resp.text().await.unwrap(),
+                });
+            }
+        };
+
+        // Try to deserialize the response.
+        let value: CalendarEvents = resp.json().await.unwrap();
+
+        Ok(value.items)
+    }
+
     /// List events on a calendar.
     pub async fn list_calendar_events(&self, calendar_id: &str, show_deleted: bool) -> Result<Vec<CalendarEvent>, APIError> {
         // Build the request.
@@ -756,6 +791,34 @@ impl GSuite {
         Ok(value.items)
     }
 
+    /// List recurring event instances.
+    pub async fn list_recurring_event_instances(&self, calendar_id: &str, recurring_event_id: &str) -> Result<Vec<CalendarEvent>, APIError> {
+        // Build the request.
+        let request = self.request(
+            CALENDAR_ENDPOINT,
+            Method::GET,
+            &format!("calendars/{}/events/{}/instances", calendar_id, recurring_event_id),
+            (),
+            Some(&[("maxResults", "2500"), ("showDeleted", "true")]),
+        );
+
+        let resp = self.client.execute(request).await.unwrap();
+        match resp.status() {
+            StatusCode::OK => (),
+            s => {
+                return Err(APIError {
+                    status_code: s,
+                    body: resp.text().await.unwrap(),
+                });
+            }
+        };
+
+        // Try to deserialize the response.
+        let value: CalendarEvents = resp.json().await.unwrap();
+
+        Ok(value.items)
+    }
+
     /// List past events on a calendar.
     pub async fn list_past_calendar_events(&self, calendar_id: &str) -> Result<Vec<CalendarEvent>, APIError> {
         // Build the request.
@@ -981,6 +1044,8 @@ pub struct CalendarEvent {
     #[serde(default, skip_serializing_if = "String::is_empty")]
     pub id: String,
     #[serde(default, skip_serializing_if = "String::is_empty")]
+    pub calendar_id: String,
+    #[serde(default, skip_serializing_if = "String::is_empty")]
     pub status: String,
     #[serde(default, skip_serializing_if = "String::is_empty", rename = "htmlLink")]
     pub html_link: String,
